@@ -16,9 +16,9 @@ use Fcntl;
 
 extends 'CHI::Driver';
 
-has 'shmkey' => (is => 'ro', isa => 'Int');
+has 'shm_key' => (is => 'ro', isa => 'Int');
 has 'shm' => (is => 'ro', builder => '_build_shm', lazy => 1);
-has 'size' => (is => 'ro', isa => 'Int', default => 8 * 1024);
+has 'shm_size' => (is => 'rw', isa => 'Int', default => 8 * 1024);
 has 'lock' => (
 	is => 'ro',
 	builder => '_build_lock',
@@ -46,11 +46,11 @@ CHI::Driver::SharedMem - Cache data in shared memory
 
 =head1 VERSION
 
-Version 0.16
+Version 0.17
 
 =cut
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 # FIXME - get the pod documentation right so that the layout of the memory
 # area looks correct in the man page
@@ -68,8 +68,9 @@ See L<IPC::SharedMem> for more information.
     use CHI;
     my $cache = CHI->new(
 	driver => 'SharedMem',
-	size => 8 * 1024,
-	shmkey => 12344321,	# Choose something unique, but the same across
+	max_size => 8 * 1024,	# Size of the cache
+	shm_size => 32 * 1024,	# Size of the shared memory area
+	shm_key => 12344321,	# Choose something unique, but the same across
 				# all caches so that namespaces will be shared,
 				# but we won't step on any other shm areas
     );
@@ -120,9 +121,15 @@ Retrieves an object from the cache
 sub fetch {
 	my($self, $key) = @_;
 
+	# open(my $tulip, '>>', '/tmp/tulip');
+	# print $tulip __LINE__, "\n";
 	$self->_lock(type => 'read');
+	# print $tulip __LINE__, "\n";
 	my $rc = $self->_data()->{$self->namespace()}->{$key};
+	# print $tulip __LINE__, "\n";
 	$self->_unlock();
+	# print $tulip __LINE__, "\n";
+	# close $tulip;
 	return $rc;
 }
 
@@ -201,18 +208,19 @@ sub get_namespaces {
 # The area must be locked by the caller
 sub _build_shm {
 	my $self = shift;
+	my $shm_size = $self->shm_size();
 
-	if((!defined($self->size())) || ($self->size() == 0)) {
+	if((!defined($shm_size)) || ($shm_size == 0)) {
 		# Probably some strange condition in cleanup
 		# croak 'Size == 0';
 		return;
 	}
-	my $shm = IPC::SharedMem->new($self->shmkey(), $self->size(), S_IRUSR|S_IWUSR);
+	my $shm = IPC::SharedMem->new($self->shm_key(), $shm_size, S_IRUSR|S_IWUSR);
 	unless($shm) {
-		$shm = IPC::SharedMem->new($self->shmkey(), $self->size(), S_IRUSR|S_IWUSR|IPC_CREAT);
+		$shm = IPC::SharedMem->new($self->shm_key(), $shm_size, S_IRUSR|S_IWUSR|IPC_CREAT);
 		unless($shm) {
-			croak "Couldn't create a shared memory area with key ",
-				$self->shmkey(), ": $!";
+			croak "Couldn't create a shared memory area of $shm_size bytes with key ",
+				$self->shm_key(), ": $!";
 			return;
 		}
 		$shm->write(pack('I', 0), 0, $Config{intsize});
@@ -227,29 +235,45 @@ sub _build_lock {
 	# open(my $fd, '<', $0) || croak("$0: $!");
 	# FIXME: make it unique for each object, not a singleton
 	$self->lock_file('/tmp/' . __PACKAGE__);
+	# open(my $tulip, '>>', '/tmp/tulip');
+	# print $tulip "build_lock\n", $self->lock_file(), "\n";
 	open(my $fd, '>', $self->lock_file()) || croak($self->lock_file(), ": $!");
 	return $fd;
 }
 
 sub _lock {
+	return;
+
 	my ($self, %params) = @_;
 
+	# open(my $tulip, '>>', '/tmp/tulip');
+	# print $tulip $params{'type'}, ' lock ', $self->lock_file(), "\n";
+	# my $i = 0;
+	# while((my @call_details = (caller($i++)))) {
+		# print $tulip "\t", $call_details[1], ':', $call_details[2], ' in function ', $call_details[3], "\n";
+	# }
 	return unless $self->lock_file();
 
 	if(my $lock = $self->lock()) {
 		flock($lock, ($params{type} eq 'read') ? Fcntl::LOCK_SH : Fcntl::LOCK_EX);
 	} else {
-		# open(my $tulip, '>>', '/tmp/tulip');
-		# print $tulip "lost lock ", $self->lock_file(), "\n";
+		# print $tulip 'lost lock ', $self->lock_file(), "\n";
 		croak('Lost lock: ', $self->lock_file());
 	}
 }
 
 sub _unlock {
+	return;
+
 	my $self = shift;
 
+	# open(my $tulip, '>>', '/tmp/tulip');
+	# print $tulip 'unlock ', $self->lock_file(), "\n";
 	if(my $lock = $self->lock()) {
 		flock($lock, Fcntl::LOCK_UN);
+	} else {
+		# print $tulip 'lost lock for unlock ', $self->lock_file(), "\n";
+		croak('Lost lock for unlock: ', $self->lock_file());
 	}
 }
 
@@ -275,22 +299,28 @@ sub _data_size {
 sub _data {
 	my($self, $h) = @_;
 
+	# open(my $tulip, '>>', '/tmp/tulip');
+	# print $tulip __LINE__, "\n";
 	if(defined($h)) {
 		my $f = JSON::MaybeXS->new()->ascii()->encode($h);
 		my $cur_size = length($f);
+		# print $tulip __LINE__, "cmp $cur_size > ", $self->size(), "\n";
+		if($cur_size > $self->shm_size()) {
+			croak("sharedmem set failed - value too large? ($cur_size bytes) > ", $self->shm_size());
+		}
 		$self->shm()->write($f, $Config{intsize}, $cur_size);
 		$self->_data_size($cur_size);
-		# open(my $tulip, '>>', '/tmp/tulip');
 		# print $tulip "set: $cur_size bytes\n";
+		# close $tulip;
 		return $h;
 	}
 	my $cur_size = $self->_data_size();
-	# open(my $tulip, '>>', '/tmp/tulip');
 	# print $tulip "get: $cur_size bytes\n";
-	unless($cur_size) {
-		return {};
+	# close $tulip;
+	if($cur_size) {
+		return JSON::MaybeXS->new()->ascii()->decode($self->shm()->read($Config{intsize}, $cur_size));
 	}
-	return JSON::MaybeXS->new()->ascii()->decode($self->shm()->read($Config{intsize}, $cur_size));
+	return {};
 }
 
 =head2 BUILD
@@ -302,9 +332,10 @@ Constructor - validate arguments
 sub BUILD {
 	my $self = shift;
 
-	unless($self->shmkey()) {
+	unless($self->shm_key()) {
 		croak 'CHI::Driver::SharedMem - no key given';
 	}
+	$| = 1;
 }
 
 =head2 DEMOLISH
@@ -315,13 +346,14 @@ it's safe to remove it and reclaim the memory.
 =cut
 
 sub DEMOLISH {
-	if(defined($^V) && ($^V ge 'v5.14.0')) {
-		return if ${^GLOBAL_PHASE} eq 'DESTRUCT';	# >= 5.14.0 only
-	}
+	# if(defined($^V) && ($^V ge 'v5.14.0')) {
+		# return if ${^GLOBAL_PHASE} eq 'DESTRUCT';	# >= 5.14.0 only
+	# }
 	my $self = shift;
 
 	# open(my $tulip, '>>', '/tmp/tulip');
-	if($self->shmkey() && $self->shm()) {
+	# print $tulip "DEMOLISH\n";
+	if($self->shm_key() && $self->shm()) {
 		my $cur_size;
 		$self->_lock(type => 'write');
 		$cur_size = $self->_data_size();
@@ -363,6 +395,7 @@ sub DEMOLISH {
 			close $self->lock();
 			unlink $lock_file;
 			# print $tulip "unlink $lock_file\n";
+			# close $tulip;
 		}
 	}
 }
@@ -377,6 +410,12 @@ Please report any bugs or feature requests to C<bug-chi-driver-sharedmem at rt.c
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=CHI-Driver-SharedMem>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
+Max_size is handled, but if you're not consistent across the calls to each cache,
+the results are unpredictable because it's used to create the size of the shared memory
+area.
+
+The shm_size argument should be deprecated and only the max_size argument used.
+
 =head1 SEE ALSO
 
 L<CHI>, L<IPC::SharedMem>
